#!/usr/bin/Rscript --vanilla

##
## Source an R file located in the same directory as this script.
##
source.local <- function(filename, arg.prefix="--file=", quiet=TRUE) {
    args <- commandArgs(trailingOnly=FALSE)
    script.arg <- grep(arg.prefix, args, value=TRUE)
    if (length(script.arg) != 1) {
        stop(paste0('unable to source "', filename, '".'))
    }
    this.file <- substring(script.arg, nchar(arg.prefix) + 1)
    source.file <- file.path(dirname(this.file), filename)
    if (! quiet) {
        cat(paste0('Sourcing "', filename, '" from "', basename(this.file),
                   '".\n'))
    }
    source(source.file)
}

##
## Print the command-line usage documentation.
##
usage <- function() {
    cat("\nUsage: plot-delay-effects.R [input files]\n\n")
    cat("Options:\n")
    cat("    --help\n")
    cat("            Show this usage information.\n")
    cat("\n")
    cat("    --out FILE\n")
    cat("            Print all plots to FILE (default: delay-effects.pdf).\n")
    cat("\n")
    return(invisible(NULL))
}

##
## Plot epidemic outcomes from a single file.
##
main <- function(args) {
    in.files <- c('png.csv')
    out.file <- 'delay-effects.pdf'

    while (length(args) >= 1 && substr(args[1], 1, 2) == "--") {
        if (args[1] == "--") {
            args <- args[-1]
            break
        } else if (args[1] == "--help") {
            return(usage())
        } else if (args[1] == "--out") {
            if (length(args) >= 2) {
                out.file <- args[2]
                args <- args[c(-1, -2)]
            } else {
                cat("Error: --out requires an argument\n")
                return(usage())
            }
        } else {
            cat(paste0("Error: unknown argument '", args[1], "'\n"))
            return(usage())
        }
    }

    ## Check whether input files were given.
    if (length(args) > 0) {
        in.files <- args
    }

    source.local("theme_rgm.R")

    df <- NULL
    for (file in in.files) {
        df_in <- read.csv(file)
        df <- rbind(df, df_in)
    }

    df$FracCtrl <- 100 * df$PrSmall / (df$PrSmall + df$PrLarge)
    df$BoostedAsc <- factor(paste0(100 * df$BoostedAsc, '%'))

    plot_theme <- theme_rgm(axis.black = TRUE, hide.title = FALSE) +
        theme(legend.position = 'right')

    p <- ggplot(df, aes(x = Delay, y = FracCtrl, colour = BoostedAsc)) +
        geom_line() +
        geom_point() +
        xlab("Delay (weeks)") +
        ylab("Controlled (%)") +
        scale_colour_brewer('Case Asc', palette = 'Set1') +
        ylim(0, 100) +
        facet_wrap(~ Country, ncol = 2) +
        plot_theme

    CairoPDF(out.file, width = 10, height = 7, bg = "transparent")
    print(p)
    invisible(dev.off())
}

##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("plot-delay-effects.R", file.arg)) {
        invisible(main(args = commandArgs(TRUE)))
    }
}
