#!/usr/bin/Rscript --vanilla

##
## Source an R file located in the same directory as this script.
##
source.local <- function(filename, arg.prefix="--file=", quiet=TRUE) {
    args <- commandArgs(trailingOnly=FALSE)
    script.arg <- grep(arg.prefix, args, value=TRUE)
    if (length(script.arg) != 1) {
        stop(paste0('unable to source "', filename, '".'))
    }
    this.file <- substring(script.arg, nchar(arg.prefix) + 1)
    source.file <- file.path(dirname(this.file), filename)
    if (! quiet) {
        cat(paste0('Sourcing "', filename, '" from "', basename(this.file),
                   '".\n'))
    }
    source(source.file)
}

##
## Print the command-line usage documentation.
##
usage <- function() {
    cat("\nUsage: plot-outcomes.R [outcomes.csv]\n\n")
    cat("Options:\n")
    cat("    --help\n")
    cat("            Show this usage information.\n")
    cat("\n")
    cat("    --out FILE\n")
    cat("            Print all plots to FILE (default: outcomes.pdf).\n")
    cat("\n")
    return(invisible(NULL))
}

##
## Plot epidemic outcomes from a single file.
##
main <- function(args) {
    in.file <- "outcomes.csv"
    out.file <- "outcomes.pdf"

    while (length(args) >= 1 && substr(args[1], 1, 2) == "--") {
        if (args[1] == "--") {
            args <- args[-1]
            break
        } else if (args[1] == "--help") {
            return(usage())
        } else if (args[1] == "--out") {
            if (length(args) >= 2) {
                out.file <- args[2]
                args <- args[c(-1, -2)]
            } else {
                cat("Error: --out requires an argument\n")
                return(usage())
            }
        } else {
            cat(paste0("Error: unknown argument '", args[1], "'\n"))
            return(usage())
        }
    }

    ## Ensure one input file is given.
    if (length(args) == 1) {
        in.file <- args[1]
    } else if (length(args) > 1)  {
        cat("Error: too many arguments\n")
        return(usage())
    }

    source.local("theme_rgm.R")

    df <- read.csv(in.file, header = TRUE, stringsAsFactors = FALSE)
    df <- reshape(df, direction = "long",
                  varying = list(c("PrDieOut", "PrSmall", "PrLarge")),
                  v.names = "Pr", timevar = "Outcome",
                  idvar = c("Country", "Preemptive", "Reactive", "PrCaseAsc"))

    outcome.lvls <- c(" Uncontrolled", " Controlled", " Die-out")
    df$Outcome[df$Outcome == 1] <- " Die-out"
    df$Outcome[df$Outcome == 2] <- " Controlled"
    df$Outcome[df$Outcome == 3] <- " Uncontrolled"
    df$Outcome <- factor(df$Outcome, ordered = TRUE, levels = outcome.lvls)
    df$Reactive[df$Reactive == "No"] <- "None"
    df$Reactive <- factor(df$Reactive, ordered = TRUE,
                          levels = c("None", "Small", "Large"))

    ## Calculate the probability of control, ignoring die out.
    pr.ctrl <- df$Pr[df$Outcome == " Controlled"]
    pr.unct <- df$Pr[df$Outcome == " Uncontrolled"]
    df$Pr2 <- df$Pr / (pr.ctrl + pr.unct)
    df$Pr2[df$Outcome == " Die-out"] <- 0

    ## Use a traffic-light colour scheme.
    clr.breaks <- levels(df$Outcome)
    clr.values <- c("#FF0000", "#FFAA00", "#006A00")

    df.nopre <- df[df$Preemptive == "No", ]
    p1 <- ggplot(df.nopre, aes(x = PrCaseAsc, y = Pr,
                               colour = Outcome, fill = Outcome)) +
        geom_bar(stat = "identity") +
        xlab("Detection Probability") +
        ylab("Frequency") +
        scale_colour_manual(breaks = clr.breaks, values = clr.values) +
        scale_fill_manual(breaks = clr.breaks, values = clr.values) +
        facet_grid(Reactive ~ Country) +
        ggtitle("No Preemptive Boost") +
        theme_rgm(axis.black = TRUE, key.box = 0.8, legend.position = "right")

    df.nopre2 <- df.nopre[df.nopre$Outcome != " Die-out", ]
    p2 <- ggplot(df.nopre2, aes(x = PrCaseAsc, y = Pr2,
                                 colour = Outcome, fill = Outcome)) +
        geom_bar(stat = "identity") +
        xlab("Detection Probability") +
        ylab("Frequency") +
        scale_colour_manual(breaks = clr.breaks, values = clr.values) +
        scale_fill_manual(breaks = clr.breaks, values = clr.values) +
        facet_grid(Reactive ~ Country) +
        ggtitle("No Preemptive Boost") +
        theme_rgm(axis.black = TRUE, key.box = 0.8, legend.position = "right")

    df.pre <- df[df$Preemptive == "Yes", ]
    p3 <- ggplot(df.pre, aes(x = PrCaseAsc, y = Pr,
                             colour = Outcome, fill = Outcome)) +
        geom_bar(stat = "identity") +
        xlab("Detection Probability") +
        ylab("Frequency") +
        scale_colour_manual(breaks = clr.breaks, values = clr.values) +
        scale_fill_manual(breaks = clr.breaks, values = clr.values) +
        facet_grid(Reactive ~ Country) +
        ggtitle("With Preemptive Boost") +
        theme_rgm(axis.black = TRUE, key.box = 0.8, legend.position = "right")

    df.pre2 <- df.pre[df.pre$Outcome != " Die-out", ]
    p4 <- ggplot(df.pre2, aes(x = PrCaseAsc, y = Pr2,
                              colour = Outcome, fill = Outcome)) +
        geom_bar(stat = "identity") +
        xlab("Detection Probability") +
        ylab("Frequency") +
        scale_colour_manual(breaks = clr.breaks, values = clr.values) +
        scale_fill_manual(breaks = clr.breaks, values = clr.values) +
        facet_grid(Reactive ~ Country) +
        ggtitle("With Preemptive Boost") +
        theme_rgm(axis.black = TRUE, key.box = 0.8, legend.position = "right")

    ## pdf(out.file, width = 10, height = 7)
    CairoPDF(out.file, width = 10, height = 7, bg = "transparent")
    print(p1)
    print(p3)
    print(p2)
    print(p4)
    invisible(dev.off())
}

##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("plot-outcomes.R", file.arg)) {
        invisible(main(args = commandArgs(TRUE)))
    }
}
