#!/usr/bin/Rscript --vanilla

##
## Source an R file located in the same directory as this script.
##
source.local <- function(filename, arg.prefix="--file=", quiet=TRUE) {
    args <- commandArgs(trailingOnly=FALSE)
    script.arg <- grep(arg.prefix, args, value=TRUE)
    if (length(script.arg) != 1) {
        stop(paste0('unable to source "', filename, '".'))
    }
    this.file <- substring(script.arg, nchar(arg.prefix) + 1)
    source.file <- file.path(dirname(this.file), filename)
    if (! quiet) {
        cat(paste0('Sourcing "', filename, '" from "', basename(this.file),
                   '".\n'))
    }
    source(source.file)
}

##
## Print the command-line usage documentation.
##
usage <- function() {
    cat("\nUsage: plot.R [options] input1.hdf5 ... inputN.hdf5\n\n")
    cat("Options:\n")
    cat("    --out FILE\n")
    cat("            Print all plots to FILE (required).\n")
    cat("\n")
    cat("    --help\n")
    cat("            Show this usage information.\n")
    cat("\n")
    return(invisible(NULL))
}

##
## Load simulation results from a single file.
##
load.results <- function(in.file, outcomes) {
    output <- h5read(in.file, '/output/final')
    patch.names <- h5read(in.file, '/params/patches/patch_names')

    uq.steps <- unique(output$step)
    if (length(uq.steps) != 1) {
        cat("Warning: ", length(uq.steps), "unique steps\n")
    }
    exp.r <- length(unique(output$model)) * length(unique(output$patch))
    if (nrow(output) != exp.r) {
        cat("Warning: expected", exp.r, "rows, found", nrow(output), "\n")
    }

    cum.i <- aggregate(output$CumI, list(model = output$model), FUN = sum)
    popn <- sum(h5read(in.file, '/params/patches/N'))
    cum.i$AR <- cum.i$x / popn

    denom <- nrow(cum.i)
    frac.die <- sum(cum.i$x < 50) / denom
    frac.sml <- sum(cum.i$x >= 50 & cum.i$AR <= 0.01) / denom
    frac.lrg <- sum(cum.i$x >= 50 & cum.i$AR > 0.01) / denom
    pcnt <- 100 * c(frac.die, frac.sml, frac.lrg)

    ## Identify the scenario by country, preemptiveness and intervention.
    ctry <- h5read(in.file, '/params/simulation/country')
    if (h5read(in.file, '/params/simulation/preemptive')) {
        preempt <- 'Pre' ## 'Preemptive'
    } else {
        preempt <- 'Reac' ## 'Reactive'
    }
    intv <- h5read(in.file, '/params/simulation/intervention')
    scen <- paste0(ctry, ": ", preempt, " ", intv)

    return(data.frame(res = outcomes, pcnt = pcnt,
                      country = ctry, intervention = intv,
                      preemptive = preempt, scenario = scen))
}

##
## Load simulation results from multiple files.
##
load.all.results <- function(in.files, outcomes) {
    df <- NULL
    for (in.file in in.files) {
        cat("Loading", in.file, "...")
        df.file <- load.results(in.file, outcomes)
        cat(" done\n")

        if (any(df.file$scenario %in% df$scenario)) {
            cat("Warning: duplicate scenarios\n")
        }

        df <- rbind(df, df.file)
    }
    return(df)
}

##
## Plot the frequency of the various epidemic outcomes.
##
plot.outcomes <- function(df, clr.breaks, clr.values) {
    p.out <- ggplot(df, aes(x = scenario, y = pcnt,
                            colour = res, fill = res)) +
        geom_bar(stat = 'identity') +
        scale_colour_manual('Outcome',
                            breaks = clr.breaks,
                            values = clr.values) +
        scale_fill_manual('Outcome',
                            breaks = clr.breaks,
                            values = clr.values) +
        xlab(NULL) +
        ylab("Frequency (%)") +
        theme_rgm()

    return(p.out)
}

##
## Plot simulation results from one or more data files.
##
main <- function(args) {
    out.file <- NULL

    while (length(args) >= 1 && substr(args[1], 1, 2) == "--") {
        if (args[1] == "--") {
            args <- args[-1]
            break
        } else if (args[1] == "--help") {
            return(usage())
        } else if (args[1] == "--out") {
            if (length(args) >= 2) {
                out.file <- args[2]
                args <- args[c(-1, -2)]
            } else {
                cat("Error: --out requires an argument\n")
                return(usage())
            }
        } else if (substring(args[1], 1, 6) == '--out=') {
            out.file <- substring(args[1], 7)
            args <- args[-1]
        } else {
            cat(paste0("Error: unknown argument '", args[1], "'\n"))
            return(usage())
        }
    }

    ## Ensure an output file was specified.
    if (is.null(out.file)) {
        cat("Error: no output file was specified\n")
        return(usage())
    }

    ## Ensure at least one input file is given.
    if (length(args) == 0) {
        cat("Error: no input files\n")
        return(usage())
    }

    suppressPackageStartupMessages(library(rhdf5))
    suppressPackageStartupMessages(library(ggplot2))
    suppressPackageStartupMessages(library(scales))
    suppressPackageStartupMessages(library(Cairo))

    source.local("theme_rgm.R")

    ## The labels for each simulation outcome.
    outcomes <- c('Die out', 'Mild', 'Severe')
    ## Ensure outcomes are listed from worst to best in the plot legend.
    clr.brks <- rev(outcomes)
    ## Custom traffic-light palette, based on the "Paired" scheme (n = 12),
    ## and interpolating between the bright orange and pale yellow colours.
    clr.vals <- c("#33a02c", "#ffaf4f", "#e31a1c")

    df <- load.all.results(args, outcomes)
    p.out <- plot.outcomes(df, clr.breaks = clr.brks, clr.values = clr.vals)

    CairoPDF(out.file, bg = "transparent")
    print(p.out)
    invisible(dev.off())
}

## Only call main() if this script is being run from the command-line.
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("plot.R", file.arg)) {
        invisible(main(args = commandArgs(TRUE)))
    }
}
