#!/usr/bin/Rscript --vanilla

peak.stats <- function(in.file, min.peak, verbose = FALSE) {
    if (verbose) { cat(in.file, "...") }
    df.all <- h5read(in.file, '/output/all')
    if (verbose) { cat("\n") }

    if (verbose) { cat("    Aggregating df.net ...\n") }
    ## Sum the time-series over all patches at each time-step (i.e., daily).
    df.net <- aggregate(df.all[! names(df.all) %in% c('step', 'patch')],
                        by = list(model = df.all$model, day = df.all$step),
                        FUN = sum)

    if (verbose) { cat("    Calculating prevalence ...\n") }
    ## Calculate the disease prevalence at each time-step.
    df.net$Prevalence <- df.net$I0c + df.net$I0h + df.net$I0t +
        df.net$Ic + df.net$Qc + df.net$Hc +
        df.net$Ih + df.net$Qh + df.net$Hh

    if (verbose) { cat("    Finding peak prevalence ...\n") }
    ## Identify the peak prevalence for each simulation.
    df.peak <- aggregate(df.net$Prevalence, by = list(model = df.net$model),
                         FUN = max)
    names(df.peak) <- c('model', 'size')
    if (verbose) { cat("    Largest peak is", max(df.peak$size), "\n") }

    if (verbose) { cat("    Removing small peaks ...\n") }
    if (sum(df.peak$size >= min.peak) == 0) {
        return(NULL)
    }
    ## Retain only the significant peaks.
    df.peak <- df.peak[df.peak$size >= min.peak, ]

    if (verbose) { cat("    Identifying peak times ...\n") }
    ## Identify the timing of each significant peak.
    df.peak$time <- 0
    for (m in df.peak$model) {
        ix <- which(df.peak$model == m)[1]
        peak.size <- df.peak$size[ix]
        burden <- df.net$Prevalence[df.net$model == m]
        df.peak$time[ix] <- which(burden == peak.size)[1]
    }

    ## Return a summary of the peak sizes and times.
    return(list(
        size.mean = mean(df.peak$size),
        size.median = median(df.peak$size),
        size.min = min(df.peak$size),
        size.max = max(df.peak$size),
        size.sd = sd(df.peak$size),
        time.mean = mean(df.peak$time),
        time.median = median(df.peak$time),
        time.min = min(df.peak$time),
        time.max = max(df.peak$time),
        time.sd = sd(df.peak$time)))
}

attack.rates <- function(min.AR, verbose = FALSE) {
    countries <- c('png')
    pr.asc <- c(0.3, 0.65)
    interventions <- c('none', 'quick', 'slow')

    df <- NULL
    for (country in countries) {
        for (pr in pr.asc) {
            if (pr == 0.3) {
                scenario <- 'react'
            } else {
                scenario <- 'preempt'
            }

            for (interv in interventions) {
                bname <- paste(country, scenario, interv, '200', pr, sep="-")
                in.file <- paste0(bname, ".hdf5")

                ## Identify the final outbreak sizes and attack rates.
                df.fnl <- h5read(in.file, '/output/final')
                ag.infs <- aggregate(df.fnl$CumI,
                                     by = list(model = df.fnl$model),
                                     FUN = sum)
                N <- sum(h5read(in.file, '/params/patches/N'))
                ag.infs$AR <- ag.infs$x / N
                ag.infs <- ag.infs[ag.infs$AR >= min.AR, ]

                if (nrow(ag.infs) > 0) {
                    df.ar <- data.frame(
                        country = country,
                        preemptive = scenario,
                        intervention = interv,
                        AR.median = median(ag.infs$AR),
                        AR.sd = sd(ag.infs$AR),
                        AR.min = min(ag.infs$AR),
                        AR.max = max(ag.infs$AR))
                    df <- rbind(df, df.ar)
                }
            }
        }
    }

    return(df)
}

compare.stats <- function(min.peak, verbose = FALSE) {
    countries <- c('png')
    pr.asc <- c(0.3, 0.65)
    interventions <- c('none', 'quick', 'slow')

    df <- NULL
    for (country in countries) {
        for (pr in pr.asc) {
            if (pr == 0.3) {
                scenario <- 'react'
            } else {
                scenario <- 'preempt'
            }

            for (interv in interventions) {
                bname <- paste(country, scenario, interv, '200', pr, sep="-")
                in.file <- paste0(bname, ".hdf5")
                pk.stats <- peak.stats(in.file, min.peak, verbose = verbose)
                if (is.null(pk.stats)) {
                    next
                }
                df.stats <- data.frame(
                    country = country,
                    preemptive = scenario,
                    intervention = interv,
                    size.median = pk.stats$size.median,
                    size.sd = pk.stats$size.sd,
                    size.min = pk.stats$size.min,
                    size.max = pk.stats$size.max,
                    time.median = pk.stats$time.median,
                    time.sd = pk.stats$time.sd,
                    time.min = pk.stats$time.min,
                    time.max = pk.stats$time.max)
                df <- rbind(df, df.stats)
            }
        }
    }

    return(df)
}

main <- function(args) {
    suppressPackageStartupMessages(library(rhdf5))
    suppressPackageStartupMessages(library(ggplot2))
    suppressPackageStartupMessages(library(Cairo))
    source("../../bin/theme_rgm.R")

    peak.file <- 'peak-stats.csv'
    ar.file <- 'attack-rates.csv'
    plot.file <- 'effects-on-peak.pdf'

    country.lvls <- c('PNG')
    preempt.lvls <- c('Reactive', 'Preemptive')
    interv.lvls <- c('None', 'Small', 'Large')

    if (file.exists(peak.file)) {
        df.stats <- read.csv(peak.file, header = TRUE)
        df.stats$country <- factor(df.stats$country, country.lvls)
        df.stats$preemptive <- factor(df.stats$preemptive, preempt.lvls)
        df.stats$intervention <- factor(df.stats$intervention, interv.lvls)
    } else {
        df.stats <- compare.stats(min.peak = 1e2, verbose = TRUE)
        write.csv(df.stats, peak.file, row.names = FALSE, quote = FALSE)
        levels(df.stats$country) <- country.lvls
        levels(df.stats$preemptive) <- preempt.lvls
        levels(df.stats$intervention) <- interv.lvls
    }

    if (file.exists(ar.file)) {
        df.ar <- read.csv(ar.file, header = TRUE)
        df.ar$country <- factor(df.ar$country, country.lvls)
        df.ar$preemptive <- factor(df.ar$preemptive, preempt.lvls)
        df.ar$intervention <- factor(df.ar$intervention, interv.lvls)
    } else {
        df.ar <- attack.rates(min.AR = 0.01)
        write.csv(df.ar, ar.file, row.names = FALSE, quote = FALSE)
        levels(df.ar$country) <- country.lvls
        levels(df.ar$preemptive) <- preempt.lvls
        levels(df.ar$intervention) <- interv.lvls
    }

    df.mitig <- merge(
        df.stats[, c('country', 'preemptive', 'intervention', 'time.median')],
        df.ar[, c('country', 'preemptive', 'intervention', 'AR.median')])
    df.mitig$time.delay <- 0
    df.mitig$AR.rel.decr <- 0
    df.mitig$AR.abs.decr <- 0
    for (country in unique(df.mitig$country)) {
        for (preempt in unique(df.mitig$preemptive)) {
            base <- df.mitig[df.mitig$country == country
                             & df.mitig$preemptive == preempt
                             & df.mitig$intervention == 'None', ]
            for (interv in unique(df.mitig$intervention)) {
                ix <- df.mitig$country == country &
                    df.mitig$preemptive == preempt &
                    df.mitig$intervention == interv
                df.mitig$time.delay[ix] <- df.mitig$time.median[ix] -
                    base$time.median
                df.mitig$AR.rel.decr[ix] <- 100 * (1 - df.mitig$AR.median[ix] /
                                                   base$AR.median)
                df.mitig$AR.abs.decr[ix] <- 100 * (base$AR.median -
                                                       df.mitig$AR.median[ix])
            }
        }
    }
    df.mitig$AR.median <- round(100 * df.mitig$AR.median, digits=2)
    df.mitig$AR.rel.decr <- round(df.mitig$AR.rel.decr, digits=2)
    df.mitig$AR.abs.decr <- round(df.mitig$AR.abs.decr, digits=2)

    ## Print a summary of the outbreak mitigation data.
    print(df.mitig[df.mitig$intervention != 'None',
                   c('country', 'preemptive', 'intervention', 'time.delay',
                     'AR.rel.decr', 'AR.abs.decr')])

    ## Save the summary data to disk.
    write.csv(df.mitig, 'effects-on-peak.csv',
              row.names = FALSE, quote = FALSE)

    ## Construct plots of the peak times and final attack rates.
    p.tm <- ggplot(df.stats,
                   aes(x = intervention, y = time.median,
                       ymin = time.median - time.sd,
                       ymax = time.median + time.sd)) +
        geom_errorbar() +
        xlab("Intervention") +
        ylab("Peak Time (days)") +
        facet_grid(country ~ preemptive, scale="free") +
        theme_rgm()

    p.ar <- ggplot(df.ar,
                   aes(x = intervention, y = 100 * AR.median,
                       ymin = 100 * (AR.median - AR.sd),
                       ymax = 100 * (AR.median + AR.sd))) +
        geom_errorbar() +
        xlab("Intervention") +
        ylab("Final Attack Rate (%)") +
        facet_grid(country ~ preemptive, scale="free") +
        theme_rgm()

    CairoPDF(file = plot.file, width = 15, height = 12)
    print(p.tm)
    print(p.ar)
    invisible(dev.off())
}

## Only call main() if this script is being run from the command-line.
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("effects-on-peak.R", file.arg)) {
        main(args = commandArgs(TRUE))
    }
}
