#!/usr/bin/Rscript --vanilla

##
## Source an R file located in the same directory as this script.
##
source.local <- function(filename, arg.prefix="--file=", quiet=TRUE) {
    args <- commandArgs(trailingOnly=FALSE)
    script.arg <- grep(arg.prefix, args, value=TRUE)
    if (length(script.arg) != 1) {
        stop(paste0('unable to source "', filename, '".'))
    }
    this.file <- substring(script.arg, nchar(arg.prefix) + 1)
    source.file <- file.path(dirname(this.file), filename)
    if (! quiet) {
        cat(paste0('Sourcing "', filename, '" from "', basename(this.file),
                   '".\n'))
    }
    source(source.file)
}

##
## Plot the epidemic outcomes for each scenario.
##
main <- function(args) {
    in.file <- "results.ssv"
    out.file <- "results.pdf"

    source.local("../../bin/theme_rgm.R")

    df0 <- read.table(in.file, header = TRUE)
    df <- df0[df0$outcome == 'rel_ctl', ]
    df$reduce_by <- 100 * (1 - df$reduce_by)
    df$reduce_pcnt <- paste0(df$reduce_by, '%')
    df$reduce_pcnt <- factor(df$reduce_pcnt, ordered = TRUE,
                             levels = paste0(rev(sort(unique(df$reduce_by))),
                                 '%'))
    df$nth_lbl <- paste0('FD = ', df$nth)
    df$nth_lbl <- factor(df$nth_lbl, ordered = TRUE,
                         levels = paste0('FD = ', sort(unique(df$nth))))

    base.file <- file.path("..", "sensitivity-first-case-and-hc-size",
                           "results.ssv")
    base <- read.table(base.file, header = TRUE)
    base <- base[base$pr_asc == 0.7 & base$hc_size == min(base$hc_size), ]
    base <- base[base$outcome == "rel_ctl", ]
    base$nth_lbl <- paste0('FD = ', base$nth)
    base$nth_lbl <- factor(base$nth_lbl, ordered = TRUE,
                           levels = levels(df$nth_lbl))

    p <- ggplot(df, aes(x = delay, y = fraction, colour = reduce_pcnt)) +
        geom_line() +
        geom_point() +
        geom_hline(data = base, mapping = aes(yintercept = 100 * fraction),
                   linetype = 'dashed') +
        scale_colour_hue(expression(Decreased ~ beta[~D])) +
        xlab("Delay (weeks)") +
        ylab("Controlled (%)") +
        ylim(0, 100) +
        facet_wrap(~ nth_lbl) +
        theme_rgm(axis.black = TRUE, hide.title = FALSE,
                  legend.position = c(1, 0.05))

    df1 <- df0[df0$outcome %in% c('die', 'ctl', 'unc')
               & df0$reduce_by == 0.25, ]
    df1$label <- paste0('FD = ', df1$nth)
    df1$label <- factor(df1$label, ordered = TRUE,
                        levels = paste0('FD = ', sort(unique(df1$nth))))

    breaks <- c('die', 'ctl', 'unc')
    labels <- paste0(' ', c('Die-out', 'Controlled', 'Uncontrolled'), '    ')
    labels <- factor(labels, levels = labels, ordered = TRUE)
    values <- c("#FFAA00", "#006A00", "#FF0000")

    p2 <- ggplot(df1, aes(x = delay, y = fraction,
                          colour = outcome, fill = outcome)) +
        geom_bar(stat = 'identity', position = 'stack', width = 0.9) +
        scale_colour_manual("", breaks = breaks, labels = labels,
                            values = values) +
        scale_fill_manual("", breaks = breaks, labels = labels,
                          values = values) +
        scale_y_continuous(breaks = c(0, 50, 100)) +
        xlab("Delay (weeks)") +
        ylab("Outcome Frequency (%)") +
        facet_wrap(~ label) +
        theme_rgm(axis.black = TRUE, hide.title = FALSE) +
        theme(legend.position = 'top')

    CairoPDF(out.file, bg = 'transparent', width = 9.8, height = 7.4)
    print(p)
    print(p2)
    invisible(dev.off())
}

##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("plot.R", file.arg)) {
        invisible(main(args = commandArgs(TRUE)))
    }
}
