#!/usr/bin/Rscript --vanilla

##
## Source an R file located in the same directory as this script.
##
source.local <- function(filename, arg.prefix="--file=", quiet=TRUE) {
    args <- commandArgs(trailingOnly=FALSE)
    script.arg <- grep(arg.prefix, args, value=TRUE)
    if (length(script.arg) != 1) {
        stop(paste0('unable to source "', filename, '".'))
    }
    this.file <- substring(script.arg, nchar(arg.prefix) + 1)
    source.file <- file.path(dirname(this.file), filename)
    if (! quiet) {
        cat(paste0('Sourcing "', filename, '" from "', basename(this.file),
                   '".\n'))
    }
    source(source.file)
}

net_cases <- function(data_dir) {
    fs <- list.files(data_dir, 'net-.*\\.ssv', full.names = TRUE)
    df <- NULL
    for (filename in fs) {
        df <- rbind(df, read.table(filename))
    }
    names(df) <- c('cases')
    return(df)
}

attack_rates <- function(data_dir) {
    fs <- list.files(data_dir, 'attack-.*\\.ssv', full.names = TRUE)
    df <- NULL
    for (filename in fs) {
        df <- rbind(df, read.table(filename))
    }
    names(df) <- c('AR')
    df$AR <- 100 * df$AR
    return(df)
}

##
## Plot the epidemic outcomes for each scenario.
##
main <- function(args) {
    in.file <- "results.ssv"
    out.file <- "results.pdf"
    data.dir <- "out"

    source.local("../../bin/theme_rgm.R")

    df <- read.table(in.file, header=TRUE)
    df$first <- paste0('FD = ', df$nth)
    df$first <- factor(df$first, ordered = TRUE,
                       levels = paste0('FD = ', sort(unique(df$nth))))
    hc_sizes <- levels(factor(df$hc_size))
    df$size <- ''
    df$size[df$hc_size == hc_sizes[1]] <- 'Small'
    df$size[df$hc_size == hc_sizes[2]] <- 'Medium'
    df$size[df$hc_size == hc_sizes[3]] <- 'Large'
    df$size <- factor(df$size, ordered = TRUE,
                      levels = c('Small', 'Medium', 'Large'))
    df$asc <- df$pr_asc * 100

    breaks <- c('die', 'ctl', 'unc')
    df <- df[df$outcome %in% breaks, ]
    labels <- paste0(' ', c('Die-out', 'Controlled', 'Uncontrolled'), '    ')
    labels <- factor(labels, levels = labels, ordered = TRUE)
    values <- c("#FFAA00", "#006A00", "#FF0000")

    p <- ggplot(df, aes(x = asc, y = fraction,
                        colour = outcome, fill = outcome)) +
        geom_bar(stat = "identity") +
        xlab("Case Ascertainment (%)") +
        ylab("Outcome Frequency (%)") +
        scale_colour_manual("", breaks = breaks, labels = labels,
                            values = values) +
        scale_fill_manual("", breaks = breaks, labels = labels,
                          values = values) +
        scale_x_continuous(breaks = c(10, 40, 70, 100)) +
        scale_y_continuous(breaks = c(0, 50, 100)) +
        facet_grid(size ~ first) +
        theme_rgm(axis.black = TRUE) +
        theme(legend.position = 'top')

    CairoPDF(out.file, bg = 'transparent', width = 9.8, height = 7.4)
    print(p)
    invisible(dev.off())

    ar <- attack_rates(data.dir)
    p <- ggplot(ar, aes(x = AR)) +
        geom_bar(binwidth = 1) +
        scale_x_continuous(breaks = c(0, 20, 40, 60, 80), limits = c(0, 80)) +
        scale_y_continuous(breaks = c()) +
        xlab("Attack Rate (%)") +
        ylab(NULL) +
        theme_rgm(axis.black = TRUE)
    p2 <- ggplot(ar, aes(x = AR)) +
        geom_bar(binwidth = 0.01) +
        scale_x_continuous(breaks = c(0, 0.2, 0.4, 0.6, 0.8, 1),
                           limits = c(0, 1)) +
        scale_y_continuous(breaks = c()) +
        xlab("Attack Rate (%)") +
        ylab(NULL) +
        theme_rgm(axis.black = TRUE)
    p3 <- ggplot(ar, aes(x = AR)) +
        geom_bar(binwidth = 0.001) +
        scale_x_continuous(limits = c(0, 0.1)) +
        scale_y_continuous(breaks = c()) +
        xlab("Attack Rate (%)") +
        ylab(NULL) +
        theme_rgm(axis.black = TRUE)
    CairoPDF('attack-rate.pdf', bg = 'transparent', width = 4.9, height = 3.6)
    print(p)
    print(p2)
    print(p3)
    invisible(dev.off())

    nc <- net_cases(data.dir)
    p <- ggplot(nc, aes(x = cases)) +
        geom_bar(binwidth = 10) +
        scale_x_continuous(limits = c(0, 1000)) +
        scale_y_continuous(breaks = c()) +
        xlab("Final Size") +
        ylab(NULL) +
        theme_rgm(axis.black = TRUE)
    p2 <- ggplot(nc, aes(x = cases)) +
        geom_bar(binwidth = 1) +
        scale_x_continuous(limits = c(0, 100)) +
        scale_y_continuous(breaks = c()) +
        xlab("Final Size") +
        ylab(NULL) +
        theme_rgm(axis.black = TRUE)
    CairoPDF('final-size.pdf', bg = 'transparent', width = 4.9, height = 3.6)
    print(p)
    print(p2)
    invisible(dev.off())
}

##
## Only call main() if this script is being run from the command-line.
##
if (! interactive()) {
    file.arg <- grep("^--file=", commandArgs(FALSE), value = TRUE)
    if (grepl("plot.R", file.arg)) {
        invisible(main(args = commandArgs(TRUE)))
    }
}
