#!/usr/bin/Rscript --vanilla

suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(themergm))
suppressPackageStartupMessages(library(Cairo))

load_from_csv <- function(csv_file, disp) {
    df0 <- read.csv(csv_file, header = TRUE, stringsAsFactors = FALSE)

    df0$Disp <- disp

    df0$Popn <- 'Rural'
    df0$Popn[df0$Region == 'Port Moresby'] <- 'Urban'

    df <- df0[((df0$Popn == 'Urban' & df0$BetaC > 0.1) |
               (df0$Popn == 'Rural' & df0$BetaC == 0.1)), ]

    outcomes <- c('PrDieOut', 'PrSmall', 'PrLarge')
    df1 <- reshape(df, varying = outcomes, times = outcomes,
                   v.names = 'Pr', timevar = 'Outcome',
                   direction = 'long')

    df1$Region <- factor(df1$Region, ordered = TRUE,
                         levels = c('Port Moresby', 'Southern', 'Highlands',
                                    'Momase', 'Islands'))
    df1$PrCaseAsc <- 100 * df1$PrCaseAsc
    df1$Pr <- 100 * df1$Pr
    df1$Outcome <- factor(df1$Outcome, ordered = TRUE, levels = outcomes)
    df1$FD_lbl <- paste0('FD = ', df1$FD_nth)
    df1$FD_lbl <- factor(df1$FD_lbl, ordered = TRUE,
                         levels = unique(paste0('FD = ', sort(df1$FD_nth))))
    df1$Int_lbl <- factor(df1$Intervention, ordered = TRUE,
                          levels = c('None', 'Small', 'Large'))
    df1$Beta_lbl <- paste0(100 * (1 - df1$ScaleBetaI), '%')

    return(df1)
}


traffic_light_fill <- function(rev = FALSE) {
    outcomes <- c('PrDieOut', 'PrSmall', 'PrLarge')
    labels <- paste0(' ', c('Die-out', 'Controlled', 'Uncontrolled'), '    ')
    labels <- factor(labels, levels = labels, ordered = TRUE)
    values <- c("#006A00", "#FFAA00", "#FF0000")
    if (rev) {
        outcomes <- rev(outcomes)
        labels <- rev(labels)
    }
    return(scale_fill_manual("", breaks = outcomes, labels = labels,
                             values = values))
}

theme_settings <- function() {
    return(theme_rgm(axis.black = TRUE))
}

plot_fd_vs_decr <- function(df, rgn = 'Port Moresby', scale = 0.75) {
    pdf <- df[df$Region == rgn
              & ((df$ScaleBetaI == 1 & df$ScaleBetaD == 1) |
                 (df$ScaleBetaI == scale & df$ScaleBetaD == 1) |
                 (df$ScaleBetaI == 1 & df$ScaleBetaD == scale) |
                 (df$ScaleBetaI == scale & df$ScaleBetaD == scale)), ]
    pdf <- pdf[pdf$Intervention == 'None', ]

    pdf$Scale_lbl <- 'Baseline'
    pdf$Scale_lbl[pdf$ScaleBetaI < 1 & pdf$ScaleBetaD == 1] <- 'Community'
    pdf$Scale_lbl[pdf$ScaleBetaD < 1 & pdf$ScaleBetaI == 1] <- 'Dead bodies'
    pdf$Scale_lbl[pdf$ScaleBetaD < 1 & pdf$ScaleBetaI < 1] <- 'Both'
    pdf$Scale_lbl <- factor(pdf$Scale_lbl, ordered = TRUE,
                            levels = c('Baseline', 'Community',
                                       'Dead bodies', 'Both'))

    p <- ggplot(pdf, aes(x = PrCaseAsc, y = Pr, fill = Outcome)) +
        geom_bar(stat = 'identity') +
        traffic_light_fill() +
        xlab('Case Ascertainment (%)') +
        ylab('Outcome Frequency (%)') +
        scale_y_continuous(breaks = c(0, 50, 100)) +
        facet_grid(Scale_lbl ~ FD_lbl) +
        theme_settings() +
        theme(legend.position = 'bottom') +
        theme(strip.text.y = element_text(colour = "black", face="bold",
                                          size = 20,
                                          angle = -90, vjust = 0.5))


    return(p)
}

df <- load_from_csv('png-patches-manuscript.csv', 0)
df$disp <- 0
for (disp in c(10, 1, 0.1, 0.01)) {
    if (disp < 1) {
        csv_file <- paste0('png-nbinom-disp-', sprintf('%0.2f', disp), '.csv')
    } else {
        csv_file <- paste0('png-nbinom-disp-', sprintf('%03d', disp), '.csv')
    }
    df_in <- load_from_csv(csv_file, disp)
    df_in$disp <- disp
    df <- rbind(df, df_in)
}

df <- df[df$Region == 'Port Moresby' & df$BetaC == 0.2
         & df$Intervention == 'None', ]

CairoPDF('port-moresby.pdf', bg = 'transparent',
         width = 9.8, height = 11.0)
for (region in sort(unique(df$Region))) {
    for (disp in unique(df$disp)) {
        if (disp == 0) {
            plot_title <- paste0(region, ' (Poisson)')
        } else {
            plot_title <- paste0(region, ' (k = ', disp, ')')
        }
        print(plot_fd_vs_decr(df[df$disp == disp, ], rgn = region) +
              ggtitle(plot_title))
    }
}
invisible(dev.off())
